pub struct EpochCommittees {
non_epoch_committee: NonEpochCommittee,
state: HashMap<EpochNumber, EpochCommittee>,
all_validators: BTreeMap<EpochNumber, RegisteredValidatorMap>,
randomized_committees: BTreeMap<EpochNumber, RandomizedCommittee<StakeTableEntry<PubKey>>>,
da_committees: BTreeMap<u64, DaCommittee>,
first_epoch: Option<EpochNumber>,
epoch_height: u64,
fixed_block_reward: Option<RewardAmount>,
fetcher: Arc<Fetcher>,
}Expand description
Type to describe DA and Stake memberships
Fields
non_epoch_committee: NonEpochCommitteeCommittee used when we’re in pre-epoch state
state: HashMap<EpochNumber, EpochCommittee>Holds Stake table and da stake
all_validators: BTreeMap<EpochNumber, RegisteredValidatorMap>holds the full validator candidate sets temporarily, until we store them
randomized_committees: BTreeMap<EpochNumber, RandomizedCommittee<StakeTableEntry<PubKey>>>Randomized committees, filled when we receive the DrbResult
da_committees: BTreeMap<u64, DaCommittee>DA committees, indexed by the first epoch in which they apply
first_epoch: Option<EpochNumber>epoch_height: u64fixed_block_reward: Option<RewardAmount>Fixed block reward (used only in V3). starting from V4, block reward is dynamic
fetcher: Arc<Fetcher>Implementations
Sourceimpl EpochCommittees
impl EpochCommittees
pub fn first_epoch(&self) -> Option<EpochNumber>
pub fn fetcher(&self) -> &Fetcher
pub fn fixed_block_reward(&self) -> Option<RewardAmount>
Sourceasync fn fetch_and_update_fixed_block_reward(
membership: Arc<RwLock<Self>>,
epoch: EpochNumber,
) -> Result<RewardAmount>
async fn fetch_and_update_fixed_block_reward( membership: Arc<RwLock<Self>>, epoch: EpochNumber, ) -> Result<RewardAmount>
Fetch the fixed block reward and update it if its None. We used a fixed block reward for version v3 Version v4 uses the dynamic block reward Assumes the stake table contract proxy address does not change
pub fn compute_block_reward( epoch: &EpochNumber, total_supply: U256, total_stake: U256, avg_block_time_ms: u64, ) -> Result<RewardAmount>
Sourcepub async fn fetch_and_calculate_block_reward(
current_epoch: EpochNumber,
coordinator: EpochMembershipCoordinator<SeqTypes>,
) -> Result<RewardAmount>
pub async fn fetch_and_calculate_block_reward( current_epoch: EpochNumber, coordinator: EpochMembershipCoordinator<SeqTypes>, ) -> Result<RewardAmount>
returns the block reward for the given epoch.
Reward depends on the epoch root header version: V3: Returns the fixed block reward as V3 only supports fixed reward
= V4 : Returns the dynamic block reward
It also attempts catchup for the root header if not present in the committee, and also for the stake table of the previous epoch before computing the dynamic block reward
Sourceasync fn calculate_dynamic_block_reward(
&self,
epoch: &EpochNumber,
header: &Header,
validators: &AuthenticatedValidatorMap,
) -> Result<Option<RewardAmount>>
async fn calculate_dynamic_block_reward( &self, epoch: &EpochNumber, header: &Header, validators: &AuthenticatedValidatorMap, ) -> Result<Option<RewardAmount>>
Calculates the dynamic block reward for a given block header within an epoch.
The reward is based on a dynamic inflation rate computed from the current stake ratio (p),
where p = total_stake / total_supply. The inflation function R(p) is defined piecewise:
- If
p <= 0.01: R(p) = 0.03 / sqrt(2 * 0.01) - Else: R(p) = 0.03 / sqrt(2 * p)
Sourcepub fn epoch_block_reward(&self, epoch: EpochNumber) -> Option<RewardAmount>
pub fn epoch_block_reward(&self, epoch: EpochNumber) -> Option<RewardAmount>
This function just returns the stored block reward in epoch committee
Sourcefn insert_committee(
&mut self,
epoch: EpochNumber,
validators: AuthenticatedValidatorMap,
block_reward: Option<RewardAmount>,
hash: Option<StakeTableHash>,
header: Option<Header>,
)
fn insert_committee( &mut self, epoch: EpochNumber, validators: AuthenticatedValidatorMap, block_reward: Option<RewardAmount>, hash: Option<StakeTableHash>, header: Option<Header>, )
Updates Self.stake_table with stake_table for
Self.contract_address at l1_block_height. This is intended
to be called before calling self.stake() so that
Self.stake_table only needs to be updated once in a given
life-cycle but may be read from many times.
pub fn active_validators( &self, epoch: &EpochNumber, ) -> Result<AuthenticatedValidatorMap>
pub fn address( &self, epoch: &EpochNumber, bls_key: BLSPubKey, ) -> Result<Address>
pub fn get_validator_config( &self, epoch: &EpochNumber, key: BLSPubKey, ) -> Result<AuthenticatedValidator<BLSPubKey>>
pub fn new_stake( committee_members: Vec<PeerConfig<SeqTypes>>, da_members: Vec<PeerConfig<SeqTypes>>, fixed_block_reward: Option<RewardAmount>, fetcher: Fetcher, epoch_height: u64, ) -> Self
pub async fn reload_stake(&mut self, limit: u64)
fn get_stake_table( &self, epoch: &Option<EpochNumber>, ) -> Option<Vec<PeerConfig<SeqTypes>>>
fn get_da_committee(&self, epoch: Option<EpochNumber>) -> DaCommittee
Sourcefn get_header(&self, epoch: EpochNumber) -> Option<&Header>
fn get_header(&self, epoch: EpochNumber) -> Option<&Header>
Get root leaf header for a given epoch